﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Optimization.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  [Item("AfterCrossoverCombinedOperator", "An operator that contains all operators that need to be executed after crossover.")]
  [StorableClass]
  public class AfterCrossoverCombinedOperator : MultiOperator<IOperator> {

    private CrossoverPerformanceAnalyzer cxAnalyzer;

    private ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }
    public IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }
    public ILookupParameter<IEvaluator> EvaluatorParameter {
      get { return ((LookupParameter<IEvaluator>)Parameters["Evaluator"]); }
    }
    public IValueParameter<SingleObjectiveSolutionSimilarityCalculator> SimilarityCalculatorParameter {
      get { return (IValueParameter<SingleObjectiveSolutionSimilarityCalculator>)Parameters["SimilarityCalculator"]; }
    }

    [StorableConstructor]
    private AfterCrossoverCombinedOperator(bool deserializing) : base(deserializing) { }
    private AfterCrossoverCombinedOperator(AfterCrossoverCombinedOperator original, Cloner cloner)
      : base(original, cloner) {
    }

    public AfterCrossoverCombinedOperator()
      : base() {
      Parameters.Add(new ScopeParameter("CurrentScope", "The current scope whose variables and sub-scopes should be removed."));
      Parameters.Add(new LookupParameter<IEvaluator>("Evaluator", "The operator which is used to evaluate new solutions."));
      Parameters.Add(new ValueParameter<SingleObjectiveSolutionSimilarityCalculator>("SimilarityCalculator"));

      SimilarityCalculatorParameter.ValueChanged += new System.EventHandler(SimilarityCalculatorParameter_ValueChanged);

      cxAnalyzer = new CrossoverPerformanceAnalyzer();
      var varDuplicator = new VariableDuplicator();

      Operators.Add(cxAnalyzer);
      Operators.Add(varDuplicator);
    }

    void SimilarityCalculatorParameter_ValueChanged(object sender, System.EventArgs e) {
      SimilarityCalculatorParameter.Value.QualityVariableName = "TSPTourLength";
      SimilarityCalculatorParameter.Value.SolutionVariableName = "TSPTour";
      cxAnalyzer.SimilarityCalculatorParameter.Value = SimilarityCalculatorParameter.Value;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new AfterCrossoverCombinedOperator(this, cloner);
    }

    public override IOperation Apply() {
      if (!Operators.Contains(EvaluatorParameter.ActualValue)) {
        Operators.Insert(0, EvaluatorParameter.ActualValue);
      }

      OperationCollection next = new OperationCollection(base.Apply());
      OperationCollection inner = new OperationCollection();
      inner.Parallel = false;

      for (int i = 0; i < Operators.Count; i++) {
        inner.Add(ExecutionContext.CreateOperation(Operators[i], CurrentScope));
      }
      next.Insert(0, inner);
      return next;
    }
  }
}
